from django.contrib.auth import get_user_model
from django.core.management.base import BaseCommand, CommandParser
from django.db.models import Q


class Command(BaseCommand):
    help = "Non-interactive user password reset"

    def add_arguments(self, parser: CommandParser):
        parser.add_argument("name", type=str)
        parser.add_argument("password", type=str)

    def handle(self, *args, **options):
        name = options["name"]
        passwd = options["password"]
        users = get_user_model().objects.filter(Q(username=name) | Q(email=name))
        if not users:
            self.stdout.write("User not found")
        for user in users:
            user.set_password(passwd)
            user.save()
            self.stdout.write("User {} password set to {}".format(name, passwd))



import os
from django.conf import settings
from django.core.management.base import CommandParser
from django.utils.html import strip_tags
from django.utils.timezone import now
from jutil.command import SafeCommand
from jutil.email import send_email, send_email_smtp, send_email_sendgrid


class Command(SafeCommand):
    help = "Sends email with (optional) attachment"

    def add_arguments(self, parser: CommandParser):
        parser.add_argument("to", type=str)
        parser.add_argument("--cc", type=str)
        parser.add_argument("--bcc", type=str)
        parser.add_argument("--sender", type=str)
        parser.add_argument("--subject", type=str)
        parser.add_argument("--body", type=str)
        parser.add_argument("--body-file", type=str)
        parser.add_argument("--attach", type=str, nargs="*")
        parser.add_argument("--smtp", action="store_true")
        parser.add_argument("--sendgrid", action="store_true")

    def do(self, *args, **kw):  # pylint: disable=too-many-branches
        files = kw["attach"] if kw["attach"] else []
        if not files:
            full_path = os.path.join(settings.BASE_DIR, "data/attachment.jpg")
            if os.path.isfile(full_path):
                files.append(full_path)
        subject = "hello " + now().isoformat()
        html = '<h1>html text</h1><p><a href="https://kajala.com/">Kajala Group Ltd.</a></p>'
        if kw["body"]:
            html = kw["body"]
        if kw["body_file"]:
            html = open(kw["body_file"], "rt", encoding="utf-8").read()  # pylint: disable=consider-using-with
        if kw["subject"]:
            subject = kw["subject"]
        text = strip_tags(html)
        sender = kw["sender"] if kw["sender"] else ""

        if kw["smtp"]:
            res = send_email_smtp(
                kw["to"],
                subject,
                text,
                html,
                sender,
                files,
                bcc_recipients=kw["bcc"],
                cc_recipients=kw["cc"],
                exceptions=True,
            )
        elif kw["sendgrid"]:
            res = send_email_sendgrid(
                kw["to"],
                subject,
                text,
                html,
                sender,
                files,
                bcc_recipients=kw["bcc"],
                cc_recipients=kw["cc"],
                exceptions=True,
            )
        else:
            res = send_email(
                kw["to"],
                subject,
                text,
                html,
                sender,
                files,
                bcc_recipients=kw["bcc"],
                cc_recipients=kw["cc"],
                exceptions=True,
            )

        self.stdout.write("send_email returned {}".format(res))



import logging
from typing import Tuple, Optional
from django.conf import settings
import requests
import socket
from ipware import get_client_ip  # type: ignore

logger = logging.getLogger(__name__)


class GeoIP:
    """
    Basic geolocation information of IP-address.
    """

    ip: str
    country_name: str
    country_code: str
    time_zone: str
    city: str
    zip_code: str
    latitude: float
    longitude: float

    def __init__(  # pylint: disable=too-many-arguments
        self,
        ip: str,
        country_name: str,
        country_code: str,
        time_zone: str,
        city: str,
        zip_code: str,
        latitude: float,
        longitude: float,
    ):
        self.ip = ip
        self.country_name = country_name
        self.country_code = country_code
        self.time_zone = time_zone
        self.city = city
        self.zip_code = zip_code
        self.latitude = latitude
        self.longitude = longitude

    def __str__(self):
        return f"{self.ip} ({self.country_code})"


def get_ip(request) -> str:
    """Returns best-guess IP for given request.
    Uses ipware library get_client_ip.
    If you need to know is IP routable or not, use ipware get_client_ip directly.
    See ipware documentation for more info.

    Note: Why such a simple function wrapper? I'm generally against wrappers like this,
    but in this case made an exceptions: I used to use ipware get_real_ip() everywhere before
    it was deprecated and had quite big update process to change all code to use ipware get_client_ip.
    I want to avoid such process again so added this wrapper.

    Args:
        request: Django's HttpRequest or DRF Request

    Returns:
        IP-address or None
    """
    return get_client_ip(request)[0]


def get_geo_ip_from_ipgeolocation(ip: str, timeout: int = 10, verbose: bool = False) -> GeoIP:
    """Returns geo IP info or empty dict if geoip query fails.
    Uses ipgeolocation.io API and requires settings.IPGEOLOCATION_API_KEY set.

    Args:
        ip: str
        timeout: timeout in seconds

    Returns:
        IPGeoInfo
    """
    if not hasattr(settings, "IPGEOLOCATION_API_KEY") or not settings.IPGEOLOCATION_API_KEY:
        raise Exception("get_geo_ip_ipstack() requires IPGEOLOCATION_API_KEY defined in Django settings")  # noqa
    url = f"https://api.ipgeolocation.io/ipgeo?apiKey={settings.IPGEOLOCATION_API_KEY}&ip={ip}"
    res = requests.get(url, timeout=timeout)
    if verbose:
        logger.info("GET %s HTTP %s: %s", url, res.status_code, res.text)
    if res.status_code != 200:
        logger.error("get_geo_ip_from_ipgeolocation(%s) failed: %s", ip, res.text)
        raise Exception("api.ipgeolocation.io HTTP {}".format(res.status_code))  # noqa
    data = res.json()
    return GeoIP(
        ip,
        country_name=data["country_name"],
        country_code=data["country_code2"],
        time_zone=(data.get("time_zone") or {}).get("name") or "",
        city=data.get("city") or "",
        zip_code=data.get("zipcode") or "",
        latitude=float(data["latitude"]),
        longitude=float(data["longitude"]),
    )


def get_geo_ip_from_ipstack(ip: str, timeout: int = 10, verbose: bool = False) -> GeoIP:
    """Returns geo IP info or empty dict if geoip query fails.
    Uses ipstack.com API and requires settings.IPSTACK_TOKEN set.

    Args:
        ip: str
        timeout: timeout in seconds

    Returns:
        IPGeoInfo
    """
    if not hasattr(settings, "IPSTACK_TOKEN") or not settings.IPSTACK_TOKEN:
        raise Exception("get_geo_ip_from_ipstack() requires IPSTACK_TOKEN defined in Django settings")  # noqa
    url = f"http://api.ipstack.com/{ip}?access_key={settings.IPSTACK_TOKEN}&format=1"
    res = requests.get(url, timeout=timeout)
    if verbose:
        logger.info("GET %s HTTP %s: %s", url, res.status_code, res.text)
    if res.status_code != 200:
        logger.error("get_geo_ip_from_ipstack(%s) failed: %s", ip, res.text)
        raise Exception("api.ipstack.com HTTP {}".format(res.status_code))  # noqa
    data = res.json()
    res_success = data.get("success", True)
    if not res_success:
        res_info = data.get("info") or ""
        logger.error("get_geo_ip_from_ipstack(%s) failed: %s", ip, res_info)
        raise Exception(res_info)  # noqa
    return GeoIP(
        ip,
        country_name=data["country_name"],
        country_code=data["country_code"],
        time_zone=data.get("time_zone") or "",
        city=data.get("city") or "",
        zip_code=data.get("zip") or "",
        latitude=float(data["latitude"]),
        longitude=float(data["longitude"]),
    )


def get_geo_ip(ip: str, timeout: int = 10, verbose: bool = False) -> GeoIP:
    """Returns geo IP info. Raises Exception if query fails.
    Uses either ipgeolocation.io (if IPGEOLOCATION_API_KEY set) or ipstack.com (if IPSTACK_TOKEN set)

    Example response (GeoIP.__dict__):

            {
                "ip": "194.100.27.41",
                "country_name": "Finland",
                "country_code": "FI",
                "time_zone": "Europe/Helsinki",
                "city": "Helsinki",
                "zip_code": "00100",
                "latitude": "60.17116",
                "longitude": "24.93265"
            }

    Args:
        ip: str
        timeout: timeout in seconds

    Returns:
        IPGeoInfo
    """
    if hasattr(settings, "IPGEOLOCATION_API_KEY") and settings.IPGEOLOCATION_API_KEY:
        return get_geo_ip_from_ipgeolocation(ip, timeout, verbose=verbose)
    if hasattr(settings, "IPSTACK_TOKEN") and settings.IPSTACK_TOKEN:
        return get_geo_ip_from_ipstack(ip, timeout, verbose=verbose)
    raise Exception("get_geo_ip() requires either IPGEOLOCATION_TOKEN or IPSTACK_TOKEN defined in Django settings")  # noqa


def get_geo_ip_or_none(ip: str, timeout: int = 10) -> Optional[GeoIP]:
    """Returns geo IP info or None if geoip query fails.
    Uses either ipgeolocation.io (if IPGEOLOCATION_API_KEY set) or ipstack.com (if IPSTACK_TOKEN set)

    Args:
        ip: str
        timeout: timeout in seconds

    Returns:
        Optional[IPGeoInfo]
    """
    try:
        return get_geo_ip(ip, timeout)
    except Exception as err:
        logger.error("get_geo_ip_or_none(%s) failed: %s", ip, err)
    return None


def get_ip_info(ip: str, exceptions: bool = False, timeout: int = 10) -> Tuple[str, str, str]:
    """Returns (ip, country_code, host) tuple of the IP address.
    Uses either ipgeolocation.io (if IPGEOLOCATION_API_KEY set) or ipstack.com (if IPSTACK_TOKEN set)

    Args:
        ip: IP address
        exceptions: Raise Exception or not
        timeout: Timeout in seconds. Note that timeout only affects geo IP part, not getting host name.

    Returns:
        (ip, country_code, host)
    """
    if not ip:  # localhost
        return "", "", ""
    host, country_code = "", ""
    try:
        geo = get_geo_ip(ip, timeout=timeout)
        country_code = geo.country_code
        if ip:
            host_info = socket.gethostbyaddr(ip)
            host = host_info[0][:255]
    except Exception as e:
        logger.error("get_ip_info(%s) failed: %s", ip, e)
        if exceptions:
            raise e
    return ip, country_code, host



# pylint: disable=too-many-branches,too-many-nested-blocks,too-many-locals
import csv
import re
from copy import copy
from typing import Any

from django.core.management.base import CommandParser
from jutil.command import SafeCommand
from jutil.bank_const_se import SE_BANK_CLEARING_LIST


def se_iban_load_map(filename: str) -> list:
    """Loads Swedish monetary institution codes in CSV format.

    Args:
        filename: CSV file name of the BIC definitions.
    Columns: Institution Name, Range Begin-Range End (inclusive), Account digits count

    Returns:
        List of (bank name, clearing code begin, clearing code end, account digits)
    """
    out = []
    name_repl = {
        "BNP Paribas Fortis SA/NV, Bankfilial Sverige": "BNP Paribas Fortis SA/NV",
        "Citibank International Plc, Sweden Branch": "Citibank",
        "Santander Consumer Bank AS (deltar endast i Dataclearingen)": "Santander Consumer Bank AS",
        "Nordax Bank AB (deltar endast i Dataclearingen)": "Nordax Bank AB",
        "Swedbank och fristående Sparbanker, t ex Leksands Sparbank och Roslagsbanken.": "Swedbank",
        "Ålandsbanken Abp (Finland),svensk filial": "Ålandsbanken Abp",
        "SBAB (deltar endast i Dataclearingen)": "SBAB Bank AB",
        "SBAB deltar endast i Dataclearingen": "SBAB Bank AB",
        "DNB Bank ASA, filial Sverige": "Den Norske Bank",
        "Länsförsäkringar Bank Aktiebolag": "Länsförsäkringar Bank AB",
        "MedMera Bank AB": "Med Mera Bank AB",
    }
    with open(filename, "rt", encoding="utf-8") as fp:
        for row in csv.reader(fp):
            if len(row) == 3:
                name, series, acc_digits = row
                # pprint([name, series, acc_digits])

                # clean up name
                name = re.sub(r"\n.*", "", name)
                name = name_repl.get(name, name)

                # clean up series
                ml_acc_digits = acc_digits.split("\n")
                for i, ser in enumerate(series.split("\n")):
                    begin, end = None, None
                    res = re.match(r"^(\d+)-(\d+).*$", ser)
                    if res:
                        begin, end = res.group(1), res.group(2)
                    if begin is None:
                        res = re.match(r"^(\d{4}).*$", ser)
                        if res:
                            begin = res.group(1)
                            end = begin

                    if begin and end:
                        digits: Any = None
                        try:
                            digits = int(acc_digits)
                        except ValueError:
                            pass
                        if digits is None:
                            try:
                                digits = int(ml_acc_digits[i])
                            except ValueError:
                                digits = "?"
                            except IndexError:
                                digits = "?"

                        out.append([name.strip(), begin.strip(), end.strip(), digits])
                        # print('OK!')
    return out


class Command(SafeCommand):
    help = "Generates Python file with Swedish bank info as constants"

    def add_arguments(self, parser: CommandParser):
        parser.add_argument("--filename", type=str)
        parser.add_argument("--php", action="store_true")

    def do(self, *args, **kw):
        new_bank_list = se_iban_load_map(kw["filename"]) if kw["filename"] else []
        # pprint(bank_list)

        bank_list = list(copy(SE_BANK_CLEARING_LIST))
        for name, begin, end, acc_digits in new_bank_list:
            exists = False
            # pylint: disable=unused-variable
            for name0, begin0, end0, acc_digits0 in bank_list:
                if begin0 == begin and end0 == end:
                    exists = True
                    break
            # pylint: enable=unused-variable
            if not exists:
                bank_list.append((name, begin, end, acc_digits))

        if kw["php"]:
            print("<?php")
            print("")
            print("global $SE_BANK_CLEARING_LIST;")
            print("$SE_BANK_CLEARING_LIST = array(")
            errors = False
            for name, begin, end, acc_digits in bank_list:
                print("    ['{}', '{}', '{}', {}],".format(name, begin, end, acc_digits))
                if acc_digits == "?":
                    errors = True
            print(");")
            if errors:
                print("")
                print('// TODO: fix errors from above marked with "?"')
            print("")
        else:
            print("SE_BANK_CLEARING_LIST = (  # " + str(len(bank_list)))
            errors = False
            for name, begin, end, acc_digits in bank_list:
                print("    ('{}', '{}', '{}', {}),".format(name, begin, end, acc_digits))
                if acc_digits == "?":
                    errors = True
            print(")")
            if errors:
                print("")
                print('# TODO: fix errors from above marked with "?"')
            print("")



from django.utils.formats import date_format


def formatted_date(description=None, ordering=None, fmt: str = "SHORT_DATE_FORMAT"):
    """Decorator for formatting date/datetime field in Django admin.

    Args:
        description: Short description of the function (default is "short date").
        ordering: Admin order field (optional).
        fmt: Format to pass to date_format (default "SHORT_DATE_FORMAT")

    Usage example in Django Admin:

    @formatted_date(_("timestamp"), "timestamp")
    def timestamp_short(obj):
        return obj.timestamp
    """

    def wrap_func(func):
        def fmt_func(*args, **kwargs):
            val = func(*args, **kwargs)
            if not val:
                return ""
            return date_format(val, fmt)

        if description is not None:
            fmt_func.short_description = description
        if ordering is not None:
            fmt_func.admin_order_field = ordering
        return fmt_func

    return wrap_func



import logging
from datetime import datetime, time, date
from typing import Optional, Any
from django.core.exceptions import ValidationError
from django.utils.translation import gettext as _
import pytz
from django.utils.dateparse import parse_datetime as django_parse_datetime
from django.utils.dateparse import parse_date as django_parse_date

logger = logging.getLogger(__name__)

TRUE_VALUES = (
    "true",
    "1",
    "yes",
    "on",
    "enabled",
)

FALSE_VALUES = (
    "false",
    "0",
    "no",
    "off",
    "disabled",
    "none",
    "null",
)


def parse_bool(v: str) -> bool:
    """
    Parses boolean value
    :param v: Input string
    :return: bool
    """
    s = str(v).lower()
    if s in TRUE_VALUES:
        return True
    if s in FALSE_VALUES:
        return False
    raise ValidationError(_("%(value)s is not one of the available choices") % {"value": v})


def parse_datetime(v: str, tz: Any = None) -> datetime:
    """Parses ISO date/datetime string to timezone-aware datetime.
    Supports YYYY-MM-DD date strings where time part is missing.
    Returns always timezone-aware datetime (assumes UTC if timezone missing).

    Args:
        v: Input string to parse
        tz: Default pytz timezone or if None then use UTC as default

    Returns:
        datetime with timezone
    """
    try:
        t = django_parse_datetime(v)
        if t is None:
            t_date: Optional[date] = django_parse_date(v)
            if t_date is None:
                raise ValidationError(_("“%(value)s” value has an invalid format. It must be in YYYY-MM-DD HH:MM[:ss[.uuuuuu]][TZ] format.") % {"value": v})
            t = datetime.combine(t_date, time())
        if tz is None:
            tz = pytz.utc
        return t if t.tzinfo else tz.localize(t)
    except Exception as err:
        raise ValidationError(_("“%(value)s” value has an invalid format. It must be in YYYY-MM-DD HH:MM[:ss[.uuuuuu]][TZ] format.") % {"value": v}) from err


def parse_bool_or_none(v: str) -> Optional[bool]:
    """Parses boolean value, or returns None if parsing fails.

    Args:
        v: Input string

    Returns:
        bool or None
    """
    s = str(v).lower()
    if s in TRUE_VALUES:
        return True
    if s in FALSE_VALUES:
        return False
    return None


def parse_datetime_or_none(v: str, tz: Any = None) -> Optional[datetime]:
    """Parses ISO date/datetime string to timezone-aware datetime.
    Supports YYYY-MM-DD date strings where time part is missing.
    Returns timezone-aware datetime (assumes UTC if timezone missing) or None if parsing fails.

    Args:
        v: Input string to parse
        tz: Default pytz timezone or if None then use UTC as default

    Returns:
        datetime with timezone or None
    """
    try:
        t = django_parse_datetime(v)
        if t is None:
            t_date: Optional[date] = django_parse_date(v)
            if t_date is None:
                return None
            t = datetime.combine(t_date, time())
        if tz is None:
            tz = pytz.utc
        return t if t.tzinfo else tz.localize(t)
    except Exception:
        return None



import mimetypes
import os
from typing import Any, List
from django.http import HttpResponse, FileResponse, Http404
from django.utils.translation import gettext as _
from jutil.format import format_xml_file, format_xml_bytes, format_csv


class FileSystemFileResponse(FileResponse):
    """File system download HTTP response.

    Args:
        full_path: Full path to file
        filename: Filename (optional) passed to client. Defaults to basename of the full path.
        disposition: Content-Disposition, default attachment; filename=xxx
    """

    def __init__(self, full_path: str, filename: str = "", disposition: str = "", **kw):
        if not os.path.isfile(full_path):
            raise Http404(_("File {} not found").format(full_path))
        if not filename:
            filename = os.path.basename(full_path)
        content_type = mimetypes.guess_type(filename)[0]
        super().__init__(open(full_path, "rb"), **kw)  # pylint: disable=consider-using-with
        if content_type:
            self["Content-Type"] = content_type
        self["Content-Length"] = os.path.getsize(full_path)
        self["Content-Disposition"] = disposition if disposition else "attachment; filename={}".format(filename)


class CsvResponse(HttpResponse):
    """CSV download HTTP response."""

    def __init__(self, rows: List[List[Any]], filename: str, dialect="excel", **kw):
        """Returns CSV response.

        Args:
            rows: List of column lists
            filename: Download response file name
            dialect: See csv.writer dialect
            **kw: Parameters to be passed to HttpResponse __init__
        """
        buf = format_csv(rows, dialect=dialect).encode("utf-8")
        super().__init__(content=buf, content_type="text/csv", **kw)
        self["Content-Disposition"] = 'attachment;filename="{}"'.format(filename)


class FormattedXmlFileResponse(HttpResponse):
    def __init__(self, filename: str):
        content = format_xml_file(filename)
        super().__init__(content)
        self["Content-Type"] = "application/xml"
        self["Content-Length"] = len(content)
        self["Content-Disposition"] = "attachment; filename={}".format(os.path.basename(filename))


class XmlResponse(HttpResponse):
    def __init__(self, content: bytes, filename: str):
        super().__init__(content)
        self["Content-Type"] = "application/xml"
        self["Content-Length"] = len(content)
        self["Content-Disposition"] = "attachment; filename={}".format(os.path.basename(filename))


class FormattedXmlResponse(XmlResponse):
    def __init__(self, content: bytes, filename: str, encoding: str = "UTF-8", exceptions: bool = True):
        super().__init__(format_xml_bytes(content, encoding=encoding, exceptions=exceptions), filename)



import json

from django.core.management.base import CommandParser
from jutil.command import SafeCommand
from jutil.files import list_files


class Command(SafeCommand):
    help = "Wrapper for testing list_files helper function output"

    def add_arguments(self, parser: CommandParser):
        parser.add_argument("dir_name", type=str)
        parser.add_argument("--suffix", type=str)
        parser.add_argument("--ignore-case", action="store_true")
        parser.add_argument("--recurse", action="store_true")
        parser.add_argument("--json", action="store_true")
        parser.add_argument("--use-media-root", action="store_true")

    def do(self, *args, **kw):
        dir_name = kw["dir_name"]
        suffix = kw["suffix"] or ""
        recurse = kw["recurse"]
        ignore_case = kw["ignore_case"]
        use_media_root = kw["use_media_root"]
        out = list_files(dir_name, suffix=suffix, ignore_case=ignore_case, use_media_root=use_media_root, recurse=recurse)
        if kw["json"]:
            json_str = json.dumps(out, indent=4)
            self.stdout.writelines([json_str])
        else:
            self.stdout.writelines(out)
